-- MultiQuestTracker (MQT) by Zerorez
MQT = {}
MQT.version = 1.3
MQT.Defaults =  {   ["position"] = { ["point"]=TOPLEFT, ["relativePoint"]=TOPLEFT, ["offsetX"]=1200, ["offsetY"]=200 },
						["bgAlpha"] = 100,
						["headerFont"] = "ZoFontHeader",
						["headerSize"] = 18,
						["qTextFont"] = "ZoFontHeader",
						["qTextSize"] = 17,
						["Tracked"] = {},
					}
local LAM = LibStub:GetLibrary("LibAddonMenu-1.0")

local bt = {}
local function MQT_DelayBuffer(key, buffer)
	if key == nil then return end
	if bt[key] == nil then bt[key] = {} end
	bt[key].buffer = buffer or 3
	bt[key].now = GetFrameTimeMilliseconds()
	if bt[key].last == nil then bt[key].last = bt[key].now end
	bt[key].diff = bt[key].now - bt[key].last
	bt[key].eval = bt[key].diff >= bt[key].buffer
	if bt[key].eval then bt[key].last = bt[key].now end
	return bt[key].eval
end

-- chain caller
local function Chain(obj)
	local T={}
	setmetatable(T,{__index = function(self,fun)
		if fun=="__END" then
			return obj
		end
		return function(self,...)
			assert(obj[fun],fun.." missing in object")
			obj[fun](obj,...)
			return self
		end
	end})
	return T
end

local WM = WINDOW_MANAGER

-- create background for MQT
MQTbg = Chain(WM:CreateControl("MQT_bg",ZO_QuestTracker,CT_STATUSBAR))
	:SetOrientation(2)
	:SetBarAlignment(1)
	:SetDimensions(242,100)
	:SetSimpleAnchorParent(0,0)
	:SetGradientColors(0,0,0,.5,0,0,0,0)
	:SetDrawLayer(1)
	:SetMouseEnabled(true)
	:SetMovable(true)
.__END

-- set proper draw layers for quest tracker
ZO_QuestTrackerContainer:SetSimpleAnchor(MQT_bg,20,15)
ZO_QuestTrackerTimerAnchor:SetAnchor(TOP,MQT_bg,BOTTOM,0,0)
ZO_QuestTrackerContainer:SetDrawLayer(2)
ZO_QuestTrackerTimerAnchor:SetDrawLayer(2)


-------------------------------------------------------
-- Updates
-------------------------------------------------------
function MQTUpdate()
	MQT_HideCheck()
	MQT_UpdateText()
end

function MQT_HideCheck()
	if not MQT_DelayBuffer("HideCheck", 100) then return; end
	local menuHidden = ZO_KeybindStripControlBackground:IsHidden()
	local interactHidden = ZO_InteractWindow:IsHidden()
	local journalHidden = ZO_QuestJournal:IsHidden()
	if (menuHidden == true) and (interactHidden == true) then
		ZO_QuestTracker:SetHidden(false)
		ZO_FocusedQuestTrackerPanel:SetHidden(true)
		ZO_QuestTrackerContainer:SetHidden(false)
		ZO_QuestTrackerTimerAnchor:SetHidden(false)
		ZO_QuestTracker:SetMovable(true)
		ZO_QuestTracker:SetMouseEnabled(true)
	elseif (menuHidden == false) or (interactHidden == false) then
		ZO_QuestTracker:SetHidden(true)
	end
	if journalHidden == false then
		MQTOnQuestJournalOpen()
	elseif journalHidden == true then
	end	
end

function MQT_UpdateText()
	if not MQT_DelayBuffer("TextUpdate", 500) then return; end
	MQT_ReloadText()
end



--------------------------------------------------
-- Quest Journal
--------------------------------------------------

function MQTToggleTracked(self, state)
	local checked
	if state == TRISTATE_CHECK_BUTTON_CHECKED then checked = true
	elseif state == TRISTATE_CHECK_BUTTON_UNCHECKED then checked = false end
	
	local pControl = self:GetParent()
	local qindex = pControl.questIndex
	local tControl = ZO_QuestTracker.tracker
	SetTracked(TRACK_TYPE_QUEST,checked,qindex)
	tControl:InitialTrackingUpdate()
	MQTHeaderCheckboxRefresh(self)
	MQTTrackingTable()
end

function MQTToggleHeaderTracked(self, state)

	for i,child in pairs(self.parent.node.children) do
		local childCB = child.control:GetNamedChild("MQT_Checkbox")
		ZO_TriStateCheckButton_SetState(childCB, state, true)
	end
	
end

function MQTGetQuestControls()
	MQT.qtable = {}
	MQT.qheaders = {}
	local qtindex = 1
	local qcontainer
	if GetNumJournalQuests == 0 then return end
	for i=1, MAX_JOURNAL_QUESTS do
		if QUEST_JOURNAL.navigationTree.rootNode.children ~= nil then
			if QUEST_JOURNAL.navigationTree.rootNode.children[i] ~= nil then
				qcontainer = QUEST_JOURNAL.navigationTree.rootNode.children[i]
				MQT.qheaders[i] = qcontainer
				for node=1, MAX_JOURNAL_QUESTS do
					if qcontainer.children[node] ~= nil then
						MQT.qtable[qtindex] = {}
						MQT.qtable[qtindex].control = qcontainer.children[node].control
						MQT.qtable[qtindex].data = qcontainer.children[node].data
						qtindex = qtindex + 1
					end
				end
			end
		end
	end
end

-- runs when quest journal is open
function MQTOnQuestJournalOpen()
	MQTGetQuestControls()
	local qindex, qcontrol, qtracked, qname
	for i=1, 30 do
		if MQT.qtable[i] ~= nil then
			qindex = MQT.qtable[i].data.questIndex
			qcontrol = MQT.qtable[i].control
			qtracked = GetIsTracked(TRACK_TYPE_QUEST,qindex)
			qname = qcontrol:GetName()
			MQTCreateCheckbox(qcontrol, qtracked, qname)
		end
	end
	
	for hindex,header in pairs(MQT.qheaders) do
		MQTCreateHeaderCheckbox(header)
	end
	
	
	if QUEST_TRACKER.assistedData ~= nil then	
		if MQT.sQuestIndex == nil then MQT.sQuestIndex = QUEST_TRACKER.assistedData:GetJournalIndex() end
		if MQT.sQuestIndex ~= QUEST_TRACKER.assistedData:GetJournalIndex() then
			MQT.sQuestIndex = QUEST_TRACKER.assistedData:GetJournalIndex()
			MQT.sCheckbox = QUEST_JOURNAL.navigationTree.selectedNode.control:GetNamedChild("MQT_Checkbox")
			MQT.sCheckbox:SetState(BSTATE_PRESSED)
			SetTracked(TRACK_TYPE_QUEST,true,MQT.sQuestIndex)
			MQT.sHeader = QUEST_TRACKER:GetHeaderForIndex(1,MQT.sQuestIndex)
			QUEST_TRACKER:SetAssisted(MQT.sHeader, true)
		end
	end
end


function MQTHeaderCheckboxFindState(header)
	local children = header.children
	local cbstate = TRISTATE_CHECK_BUTTON_UNCHECKED

	local tracked = 0
	for id,node in pairs(children) do
		local childCheckbox = node.control:GetNamedChild("MQT_Checkbox")
		local isChecked = ZO_CheckButton_IsChecked(childCheckbox)
		if isChecked == true then tracked = tracked + 1 end
	end
	if tracked == #children then  cbstate = TRISTATE_CHECK_BUTTON_CHECKED
	elseif tracked > 0 and tracked < #children then cbstate = TRISTATE_CHECK_BUTTON_INDETERMINATE end
	
	return cbstate
end

function MQTCreateHeaderCheckbox(header)
	if header.control:GetNamedChild("MQT_Checkbox") ~= nil then return end
	local cname = header.control:GetName().."MQT_Checkbox"
	local cbstate = MQTHeaderCheckboxFindState(header)
	
	
	MQThcb = Chain(WM:CreateControlFromVirtual(cname,header.control,"ZO_TriStateCheckButton"))
		:SetAnchor(RIGHT,header.control,LEFT,-2,-2)
		:SetDimensions(20,20)
		:SetHidden(false)
	.__END
	MQThcb.parent = header.control
	ZO_TriStateCheckButton_SetStateChangeFunction(MQThcb, MQTToggleHeaderTracked)
	ZO_TriStateCheckButton_SetState(MQThcb, cbstate, false)
	if header.control == ZO_QuestJournalNavigationContainerScrollChildZO_QuestJournalHeader1 then
		ZO_QuestJournalNavigationContainerScrollChildZO_QuestJournalHeader1:SetAnchor(3,ZO_QuestJournalNavigationContainerScrollChildContainer1,3,22,0)
	end
end



-- create check boxes in journal
function MQTCreateCheckbox(qcontrol, qtracked, qname)
	if qcontrol:GetNamedChild("MQT_Checkbox") ~= nil then return end
	local cbState
	if qtracked == true then
		cbState = TRISTATE_CHECK_BUTTON_CHECKED
	else
		cbState = TRISTATE_CHECK_BUTTON_UNCHECKED
	end

	local cname = tostring( qname .. "MQT_Checkbox" )
	
	MQTcb = Chain(WM:CreateControlFromVirtual(cname,qcontrol,"ZO_TriStateCheckButton"))
		:SetAnchor(RIGHT,qcontrol,LEFT,-22,0)
		:SetDimensions(18,18)
		:SetHidden(false)
	.__END
	ZO_TriStateCheckButton_SetStateChangeFunction(MQTcb, MQTToggleTracked)
	ZO_TriStateCheckButton_SetState(MQTcb, cbState, false)
	MQTcb.parent = qcontrol
	local icon = qcontrol:GetNamedChild("AssistedIcon")
	icon:SetAnchor(8,qcontrol,2,4,0)
end


function MQTHeaderCheckboxRefresh(checkbox)
	local header = checkbox.parent.node.parentNode
	local headerCheckbox = header.control:GetNamedChild("MQT_Checkbox")
	local cbstate = MQTHeaderCheckboxFindState(header)
	ZO_TriStateCheckButton_SetState(headerCheckbox, cbstate, false)
end

------------------------------------------------------
-- Quest Tables
------------------------------------------------------
function MQTTrackingTable()
	MQT.SV.Tracked = {}
	for i=1, MAX_JOURNAL_QUESTS do
		if IsValidQuestIndex(i) then
			MQT.SV.Tracked[i] = GetIsTracked(TRACK_TYPE_QUEST,i,0)
		end
	end
end

function MQT_GetTrackerEntries()
	local qControls = {}
	qControls.objectives = {}
	qControls.headers = {}
	qControls.other = {}
	
	for i,v in pairs(ZO_QuestTracker.tracker.headerPool.m_Active) do
		table.insert(qControls.headers, v)
	end
	
	for i,v in pairs(ZO_QuestTracker.tracker.conditionPool.m_Active) do
		table.insert(qControls.objectives, v)
	end

	return qControls
end

-------------------------------------------------------
-- tracker reloading functions
--------------------------------------------------------

function MQT_ReloadText()
	local qControls = MQT_GetTrackerEntries()
	for i,v in pairs(qControls.headers) do
		if (MQT.SV and (MQT.SV.headerFont and MQT.SV.headerSize)) then
			local control = GetControl(MQT.SV.headerFont)
			local font,size,style = control:GetFontInfo()
			v:SetFont(font..'|'..MQT.SV.headerSize..'|'..style)
		end
	end
	for i,v in pairs(qControls.objectives) do
		if (MQT.SV and (MQT.SV.qTextFont and MQT.SV.qTextSize)) then
			local control = GetControl(MQT.SV.qTextFont)
			local font,size,style = control:GetFontInfo()
			v:SetFont(font..'|'..MQT.SV.qTextSize..'|'..style)
		end
	end
end


-------------------------------------------------------
-- Options
-------------------------------------------------------
function MQT_bgGetAlpha()
	return MQT.SV.bgAlpha
end

function MQT_bgSetAlpha(newAlpha)
	local newAlphaPercent = newAlpha / 100
	MQT_bg:SetAlpha(newAlphaPercent)
	MQT.SV.bgAlpha = newAlpha
end


---------------------------------------
function MQT_GetHeaderFont()
	return MQT.SV.headerFont
end

function MQT_SetHeaderFont(newFont)
	MQT.SV.headerFont = newFont
	MQT_ReloadText()
end


function MQT_GetHeaderStyle()
	return MQT.SV.headerStyle
end

function MQT_SetHeaderStyle(newStyle)
	MQT.SV.headerStyle = newStyle
	MQT_ReloadText()
end


function MQT_GetHeaderSize()
	return MQT.SV.headerSize
end

function MQT_SetHeaderSize(newSize)
	MQT.SV.headerSize = newSize
	MQT_ReloadText()
end



-------------------------------------
function MQT_GetQTextFont()
	return MQT.SV.qTextFont
end

function MQT_SetQTextFont(newFont)
	MQT.SV.qTextFont = newFont
	MQT_ReloadText()
end


function MQT_GetQTextStyle()
	return MQT.SV.qTextStyle
end

function MQT_SetQTextStyle(newStyle)
	MQT.SV.qTextStyle = newStyle
	MQT_ReloadText()
end


function MQT_GetQTextSize()
	return MQT.SV.qTextSize
end

function MQT_SetQTextSize(newSize)
	MQT.SV.qTextSize = newSize
	MQT_ReloadText()
end




----------------------------------------------------------
-- On Addon Load
----------------------------------------------------------
function MQTLoadVars()
	if MQT.SV == nil then return end
	if MQT.SV.Tracked ~= nil then
		for k, v in pairs(MQT.SV.Tracked) do
			SetTracked(TRACK_TYPE_QUEST,v,k,0)
		end
	end
	
	ZO_QuestTracker.tracker:InitialTrackingUpdate()
	
	if MQT.SV.position ~= nil then
		MQT_bg:ClearAnchors()
		MQT_bg:SetAnchor(MQT.SV.position.point, GuiRoot, MQT.SV.position.relativePoint, MQT.SV.position.offsetX, MQT.SV.position.offsetY)
		MQT_bg:SetMovable(true)
	end
	
	if MQT.SV.bgAlpha ~= nil then MQT_bgSetAlpha(MQT.SV.bgAlpha) end
	
	MQT_ReloadText()
end

-- OnInit...
function MQTOnInit(eventCode, addOnName)
	if addOnName ~= "MultiQuestTracker" then return end
	
	--       ZO_SavedVars:New(savedVariableTable, version, namespace, defaults, profile, displayName, characterName)
	MQT.SV = ZO_SavedVars:New( "MQT_SV" , MQT.version, nil , MQT.Defaults, nil, GetDisplayName(), GetUnitName('player'))
	
	MQTLoadVars()
	
	MQT_bg:SetHandler("OnMouseUp", function(self) 
		MQT.SV.position.offsetX = MQT_bg:GetLeft()
		MQT.SV.position.offsetY = MQT_bg:GetTop()
		end)
	
	
	
	MQTpanel = LAM:CreateControlPanel("MQT_OptionsPanel", "MQT Settings")
		LAM:AddHeader(MQTpanel, "MQT_bgHeader", "Background ")	
			LAM:AddSlider(MQTpanel, "MQT_Option_BGAlpha", "Background Alpha", "Change the transparency of the background", 0, 100,1, MQT_bgGetAlpha, MQT_bgSetAlpha)
		LAM:AddHeader(MQTpanel, "MQT_HeaderHeader", "Quest Names")
			LAM:AddDropdown(MQTpanel, "MQT_Option_HeaderFont", "Name Font", "Set the font for the quest names", MQT_ZoFontList, MQT_GetHeaderFont, MQT_SetHeaderFont)
			LAM:AddSlider(MQTpanel, "MQT_Option_HeaderSize", "Name Font Size", "Change the font size for quest names", 8, 45,1, MQT_GetHeaderSize, MQT_SetHeaderSize)
		LAM:AddHeader(MQTpanel, "MQT_QTextHeader", "Objective Text")
			LAM:AddDropdown(MQTpanel, "MQT_Option_QTextFont", "Objective Font", "Set the font for the quest text", MQT_ZoFontList, MQT_GetQTextFont, MQT_SetQTextFont)
			LAM:AddSlider(MQTpanel, "MQT_Option_QTextSize", "Objective Font Size", "Change the font size for quest text", 8, 45,1, MQT_GetQTextSize, MQT_SetQTextSize)
		LAM:AddDescription(MQTpanel, "MQT_Option_MoveText", "Drag tracker with the mouse to move (default [period] key)")
end


EVENT_MANAGER:RegisterForEvent( "MQT", EVENT_ADD_ON_LOADED , MQTOnInit )
EVENT_MANAGER:RegisterForEvent( "MQTZoneChange1", EVENT_LINKED_WORLD_POSITION_CHANGED, MQTLoadVars )
EVENT_MANAGER:RegisterForEvent( "MQTZoneChange2", EVENT_ZONE_CHANGED, MQTLoadVars )
